/*
 * The MIT License (MIT)
 * Copyright © 2013 Englishtown <opensource@englishtown.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the “Software”), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package com.englishtown.vertx.clients;

import com.couchbase.client.CouchbaseClient;
import com.google.common.util.concurrent.FutureCallback;
import net.spy.memcached.CachedData;
import net.spy.memcached.internal.GetFuture;
import net.spy.memcached.ops.GetOperation;
import net.spy.memcached.ops.Operation;
import net.spy.memcached.ops.OperationStatus;
import net.spy.memcached.transcoders.Transcoder;

import java.io.IOException;
import java.net.URI;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.Future;

/**
 * We've overridden the CouchbaseClient so that we can add an async read that includes a callback handler.
 */
public class EtCouchbaseClient extends CouchbaseClient {
    public EtCouchbaseClient(List<URI> baseList, String bucketName, String pwd) throws IOException {
        super(baseList, bucketName, pwd);    //To change body of overridden methods use File | Settings | File Templates.
    }

    public <T> GetFuture<T> asyncGet(final String key, final Transcoder<Object> tc, final FutureCallback<T> callback) {
        final CountDownLatch latch = new CountDownLatch(1);
        final GetFuture<T> rv = new GetFuture<>(latch, operationTimeout, key);
        Operation op = opFact.get(key, new GetOperation.Callback() {
            private Future<T> val = null;

            public void receivedStatus(OperationStatus status) {
                rv.set((Future<T>) val, status);
            }

            public void gotData(String k, int flags, byte[] data) {
                assert key.equals(k) : "Wrong key returned";
                val = (Future<T>) tcService.decode(tc, new CachedData(flags, data, tc.getMaxSize()));
            }

            public void complete() {
                latch.countDown();
                if (callback != null) {
                    try {
                        callback.onSuccess((val == null ? null : val.get()));
                    } catch (Throwable t) {
                        callback.onFailure(t);
                    }
                }
            }
        });
        rv.setOperation(op);
        mconn.enqueueOperation(key, op);
        return rv;
    }

    public <T> GetFuture<T> asyncGet(final String key, final FutureCallback<T> callback) {
        return asyncGet(key, transcoder, callback);
    }
}
